@@ -13,7 +13,7 @@ class TestPeriodicJob(PeriodicJob):
 
 
 class TestPeriodic(TestPeriodicJob):
-    run_every = timedelta(seconds=10)
+    run_every = timedelta(seconds=5)
     def __call__(self, consumer):
         return 'OK'
     
@@ -39,7 +39,7 @@ class StandardDeviation(TestPeriodicJob):
         if inputs is None:
             for n in range(sample):
                 inputs = [random() for i in range(size)]
-                self.send_to_queue(consumer, self.name, inputs=inputs)
+                self.run_job(consumer, self.name, inputs=inputs)
             return 'produced %s new tasks' % sample
         else:
             time.sleep(0.1)
