@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -
 '''Event driven concurrent framework for Python'''
-VERSION = (0, 4, 6, 'final', 0)
+VERSION = (0, 5, 0, 'final', 1)
 
 from .utils.version import get_version
 
@@ -31,18 +32,27 @@ CLASSIFIERS  = ['Development Status :: 4 - Beta',
                 'Topic :: Internet :: WWW/HTTP :: WSGI :: Server',
                 'Topic :: Internet :: WWW/HTTP :: Dynamic Content']
 
-from .utils.log import *
 
 DEFAULT_PORT = 8060
 ASYNC_TIMEOUT = None
+SERVER_NAME = 'Pulsar'
+JAPANESE = b'\xe3\x83\x91\xe3\x83\xab\xe3\x82\xb5\xe3\x83\xbc'.decode('utf-8')
+CHINESE = b'\xe8\x84\x89\xe5\x86\xb2\xe6\x98\x9f'.decode('utf-8')
 SERVER_SOFTWARE = "python-{0}/{1}".format(SERVER_NAME, version)
 
 from .utils.exceptions import *
-from .utils.sock import *
 from .utils import system
-from .utils.httpurl import to_string, native_str, to_bytes
+from .utils.sockets import create_socket, parse_address
+from .utils.pep import to_string, native_str, to_bytes
 platform = system.platform
 
 from .utils.config import *
 from .async import *
 from .apps import *
+#
+# Import pubsub local backend for commands
+from .apps.pubsub import local
+del local
+# Import tasks local backend for commands
+from .apps.tasks.backends import local
+del local
