@@ -1,3 +1,12 @@
+'''
+A test plugin for profiling test cases and generating Html reports. It uses
+the ``cProfiler`` from the standard libraries.
+
+.. autoclass:: Profile
+   :members:
+   :member-order: bysource
+   
+'''
 import os
 import re
 import time
@@ -70,7 +79,7 @@ def data_stream(lines, num=None):
             for f in fields[1:-1]:
                 try:
                     float(f)
-                except:
+                except Exception:
                     valid = False
                     break
                 new_fields.append(f)
@@ -98,9 +107,11 @@ def copy_file(filename, target, context=None):
 
 
 class Profile(test.TestPlugin):
-    ''':class:`pulsar.apps.test.Plugin` for profiling test cases.'''
+    ''':class:`pulsar.apps.test.TestPlugin` for profiling test cases.'''
     desc = '''Profile tests using the cProfile'''
-    profile_stats_path = pulsar.Setting(default='htmlprof',
+    profile_stats_path = pulsar.Setting(
+                                flags=['--profile-stats-path'],
+                                default='htmlprof',
                                 desc='''location of profile directory''',
                                 validator=absolute_file)
 
@@ -118,7 +129,7 @@ class Profile(test.TestPlugin):
             local.tmp = tempfile.mktemp(dir=self.profile_temp_path)
             local.prof.enable()
         
-    def after_test_function_run(self, test, local, result):
+    def after_test_function_run(self, test, local, result, async):
         if self.config.profile:
             local.prof.disable()
             local.prof.dump_stats(local.tmp)
@@ -160,7 +171,7 @@ class Profile(test.TestPlugin):
                     bits = line.split(' ')
                     try:
                         ncalls = int(bits[0])
-                    except:
+                    except Exception:
                         continue
                     else:
                         run_info += ' ' + line
