@@ -1,3 +1,21 @@
+'''
+Plugin
+~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: Plugin
+   :members:
+   :member-order: bysource
+   
+
+Test Runner
+~~~~~~~~~~~~~~~~~~~
+
+.. autoclass:: TestRunner
+   :members:
+   :member-order: bysource
+
+   
+'''
 import sys
 import traceback
 import unittest
@@ -5,7 +23,6 @@ import logging
 from inspect import istraceback
 from copy import deepcopy
 
-from pulsar import HaltServer
 from pulsar.utils.structures import AttributeDictionary
 
 from .utils import TestFunction
@@ -26,17 +43,19 @@ STDERR_LINE = '\nStderr:\n%s'
 class Plugin(object):
     '''Interface for all classes which are part of of the :class:`TestRunner`,
 including :class:`TestRunner` itself, :class:`TestResult`
-and :class:`Plugin`.'''
+and :class:`pulsar.apps.test.TestPlugin`.'''
     descriptions = None
 
     def configure(self, cfg):
-        '''Configure the *instance*. This method is called once just after
-construction of a :class:`TestRunner` and can be used to configure the
-plugin. If it returns something other than ``None``
-(for example an abort message)
+        '''Called once just after construction of
+a :class:`TestRunner` and therefore **before any test class is loaded**.
+This is a chance to configure the :class:`Plugin` or global variables
+which may affect the way tests are run.
+If it returns something other than ``None`` (for example an abort message)
 it will stop the configuration of all subsequent plugins and quit the test.
 
-:parameter cfg: instance of :class:`pulsar.Config`.
+:parameter cfg: a :class:`pulsar.utils.config.Config`.
+:return: ``None`` unless the tests runner must be stopped.
 '''
         pass
     
@@ -44,25 +63,28 @@ it will stop the configuration of all subsequent plugins and quit the test.
     def name(self):
         return self.__class__.__name__.lower()
 
-    def loadTestsFromTestCase(self, cls):
-        pass
-
     def on_start(self):
-        '''Called once by :class:`TestSuite` just before it starts running tests.'''
+        '''Called once by :class:`TestSuite` after all tests are loaded but
+but before it starts running them.'''
         pass
 
     def on_end(self):
         '''Called once by :class:`TestSuite` just before it stops.'''
         pass
 
+    def loadTestsFromTestCase(self, test_class):
+        '''Called just before a ``test_class`` executes all its test
+methods. This is run just before ``setUpClass`` method.'''
+        pass
+    
     def startTest(self, test):
-        '''Called just before a *test* is executed. This is run
-just before _pre_setup function.'''
+        '''Called just before a ``test`` function is executed. This is run
+just before ``_pre_setup`` method.'''
         pass
 
     def stopTest(self, test):
-        '''Called just after a *test* has finished. This is run just after
-the _post_teardown function.'''
+        '''Called just after a ``test`` function has finished. This is run just
+after the ``_post_teardown`` method.'''
         pass
     
     def before_test_function_run(self, test, local):
@@ -70,7 +92,7 @@ the _post_teardown function.'''
 behaviour in the process domain where the test run.'''
         return test
     
-    def after_test_function_run(self, test, local, result):
+    def after_test_function_run(self, test, local, result, async):
         '''Given a test-function instance return a, possibly, modified test
 instance. This function can be used by plugins to modify the behaviour of test
 cases. By default it returns *test*.'''
@@ -99,7 +121,7 @@ cases. By default it returns *test*.'''
 
     def getDescription(self, test):
         doc_first_line = test.shortDescription()
-        teststr = '{0}.{1}'.format(test.tag,test)
+        teststr = '%s.%s' % (test.tag, test)
         if self.descriptions and doc_first_line:
             return '\n'.join((teststr, doc_first_line))
         else:
@@ -138,37 +160,38 @@ class TestStream(TestResultProxy):
         self.dots = verbosity == 1
 
     def handler(self, name):
-        return self._handlers.get(name,self.stream)
+        return self._handlers.get(name, self.stream)
 
     def startTest(self, test):
         if self.showAll:
-            v = self.getDescription(test) + ' ... '
-            self.stream.write(v)
-            self.stream.flush()
+            self.head(test, 'Started')
+            
+    #def after_test_function_run(self, test, local, result, async):
+    #    if async:
+    #        self.startTest(test)
 
     def head(self, test, v):
-        #v = self.getDescription(test) + ' ... ' + v + '\n'
-        v += '\n'
+        v = self.getDescription(test) + ' ... %s\n' % v
         self.stream.write(v)
         self.stream.flush()
 
     def addSuccess(self, test):
         if self.showAll:
-            self.head(test, 'ok')
+            self.head(test, 'OK')
         elif self.dots:
             self.stream.write('.')
             self.stream.flush()
 
     def addError(self, test, err):
         if self.showAll:
-            self.head(test,"ERROR")
+            self.head(test, 'ERROR')
         elif self.dots:
             self.stream.write('E')
             self.stream.flush()
 
     def addFailure(self, test, err):
         if self.showAll:
-            self.head(test,"FAIL")
+            self.head(test, "FAIL")
         elif self.dots:
             self.stream.write('F')
             self.stream.flush()
@@ -182,14 +205,14 @@ class TestStream(TestResultProxy):
 
     def addExpectedFailure(self, test, err):
         if self.showAll:
-            self.head(test,"expected failure")
+            self.head(test, "expected failure")
         elif self.dots:
             self.stream.write("x")
             self.stream.flush()
 
     def addUnexpectedSuccess(self, test):
         if self.showAll:
-            self.head(test,"unexpected success")
+            self.head(test, "unexpected success")
         elif self.dots:
             self.stream.write("u")
             self.stream.flush()
@@ -249,7 +272,7 @@ class TestStream(TestResultProxy):
 
 
 class TestResult(Plugin):
-
+    '''A result of a test run.'''
     def __init__(self, descriptions=True):
         self.descriptions = descriptions
         self.testsRun = 0
@@ -352,9 +375,7 @@ def testsafe(name, return_val=None):
                 c = getattr(p, name)(*args)
                 if c:
                     return return_val(c)
-            except HaltServer:
-                raise
-            except:
+            except Exception:
                 LOGGER.critical('Unhadled error in %s.%s' % (p, name),
                                 exc_info=True)
     return _                
@@ -393,11 +414,11 @@ class TestRunner(TestResultProxy):
     printErrors = testsafe('printErrors')
     printSummary = testsafe('printSummary')
 
-    def loadTestsFromTestCase(self, cls):
+    def loadTestsFromTestCase(self, test_cls):
         '''Load all *test* functions for the test class *cls*.'''
-        c = testsafe('loadTestsFromTestCase', lambda v: v)(self, cls)
+        c = testsafe('loadTestsFromTestCase', lambda v: v)(self, test_cls)
         if c is None:
-            return self.loader.loadTestsFromTestCase(cls)
+            return self.loader.loadTestsFromTestCase(test_cls)
         else:
             return c
 
@@ -420,19 +441,20 @@ class TestRunner(TestResultProxy):
             test = p.before_test_function_run(test, local) or test
         return test
     
-    def after_test_function_run(self, test, result):
+    def after_test_function_run(self, test, result, async):
         '''Called before the test starts.'''
         for p in self.plugins:
             local = test.plugins.get(p.name)
             if local is not None:
-                p.after_test_function_run(test, local, result)
+                p.after_test_function_run(test, local, result, async)
         return result
             
-    def run_test_function(self, test, func):
+    def run_test_function(self, test, func, timeout=None):
         '''Run function *func* which belong to *test*.
     
 :parameter test: test instance or class
 :parameter func: test function belonging to *test*
+:parameter timeout: An optional timeout for asynchronous tests.
 :return: an asynchronous result
 '''
         if func is None:
@@ -440,4 +462,4 @@ class TestRunner(TestResultProxy):
         tfunc = getattr(func, 'testfunction', None)
         if tfunc is None:
             tfunc = TestFunction(func.__name__)
-        return tfunc(test)
\ No newline at end of file
+        return tfunc(test, timeout)
\ No newline at end of file
