@@ -2,18 +2,17 @@ import time
 from multiprocessing import Process, current_process
 from threading import Thread, current_thread
 
-from pulsar import system, wrap_socket, platform, socket_pair
+from pulsar import system, platform
 from pulsar.utils.security import gen_unique_id
 
-from .iostream import AsyncIOStream
 from .proxy import ActorProxyMonitor, get_proxy
-from .defer import pickle, EXIT_EXCEPTIONS
 from .access import get_actor, get_actor_from_id
+from .threads import KillableThread
 
 
 __all__ = ['Concurrency', 'concurrency']
 
-def concurrency(kind, actor_class, monitor, commands_set, cfg, **params):
+def concurrency(kind, actor_class, monitor, cfg, **params):
     '''Function invoked by the :class:`Arbiter` or a :class:`Monitor` when
 spawning a new :class:`Actor`. It created a :class:`Concurrency` instance
 which handle the contruction and the lif of an :class:`Actor`.
@@ -30,36 +29,34 @@ which handle the contruction and the lif of an :class:`Actor`.
         c = ActorProcess()
     else:
         raise ValueError('Concurrency %s not supported in pulsar' % kind)
-    return c.make(kind, actor_class, monitor, commands_set, cfg, **params)
+    return c.make(kind, actor_class, monitor, cfg, **params)
 
 
 class Concurrency(object):
-    '''Actor implementation is responsible for the actual spawning of
+    '''Actor :class:`Concurrency` is responsible for the actual spawning of
 actors according to a concurrency implementation. Instances are pickable
 and are shared between the :class:`Actor` and its
 :class:`ActorProxyMonitor`.
 
 :parameter concurrency: string indicating the concurrency implementation.
-    Valid choices are ``monitor``, ``process`` and ``thread``.
+    Valid choices are ``monitor``, ``process``, ``thread``.
 :parameter actor_class: :class:`Actor` or one of its subclasses.
 :parameter timeout: timeout in seconds for the actor.
 :parameter kwargs: additional key-valued arguments to be passed to the actor
     constructor.
 '''
     _creation_counter = 0
-    address = None
-    def make(self, kind, actor_class, monitor, commands_set, cfg, name=None,
-             aid=None, **params):
+    def make(self, kind, actor_class, monitor, cfg, name=None, aid=None, **kw):
         self.__class__._creation_counter += 1
         self.aid = aid or gen_unique_id()[:8]
         self.age = self.__class__._creation_counter
         self.name = name or actor_class.__name__.lower()
         self.kind = kind
-        self.commands_set = commands_set
         self.cfg = cfg
         self.actor_class = actor_class
-        self.params = params
-        return self.get_actor(monitor)
+        self.params = kw
+        self.params['monitor'] = monitor
+        return self.get_actor()
 
     @property
     def unique_name(self):
@@ -69,23 +66,17 @@ and are shared between the :class:`Actor` and its
         return self.unique_name
     __str__ = __repr__
 
-    def get_actor(self, monitor):
+    def get_actor(self):
         self.daemon = True
-        if monitor.is_arbiter():
-            arbiter = monitor
-        else:
-            arbiter = monitor.arbiter
-        self.params['arbiter'] = get_proxy(arbiter)
-        self.params['monitor'] = get_proxy(monitor)
+        self.params['monitor'] = get_proxy(self.params['monitor'])
         return ActorProxyMonitor(self)
-                
 
 
 class MonitorConcurrency(Concurrency):
-    '''An actor implementation for Monitors. Monitors live in the main process
-loop and therefore do not require an inbox.'''
-    def get_actor(self, arbiter):
-        self.params['arbiter'] = arbiter
+    ''':class:`Concurrency` class for monitors such as the :class:`Arbiter`
+and :class:`Monitor`. Monitors live in the **mainthread** of the master process
+and therefore do not require to be spawned.'''
+    def get_actor(self):
         return self.actor_class(self)
 
     def start(self):
@@ -100,31 +91,26 @@ loop and therefore do not require an inbox.'''
 
 
 class ActorConcurrency(Concurrency):
-
+    '''Base class for all :class:`Actor` concurrency models. Must implement
+the **start** method.'''
     def run(self):
-        try:
-            actor = self.actor_class(self)
-            actor.start()
-        except EXIT_EXCEPTIONS:
-            # This is needed in windows in order to avoid useless traceback
-            # on KeyboardInterrupt
-            pass
+        actor = self.actor_class(self)
+        actor.start()
 
 
 class ActorProcess(ActorConcurrency, Process):
     '''Actor on a process'''
-    pass
 
 
-class ActorThread(ActorConcurrency, Thread):
+class ActorThread(ActorConcurrency, KillableThread):
     '''Actor on a thread'''
-    def terminate(self):
-        '''Called by the main thread to force termination.'''
-        actor = get_actor_from_id(self.aid)
-        if actor is not None:
-            actor.exit()
-
-    @property
-    def pid(self):
-        return current_process().pid
 
+    def terminate(self, kill=False):
+        if self.is_alive():
+            actor = get_actor_from_id(self.aid)
+            me = get_actor()
+            if not kill and actor and actor != me:
+                actor.stop(1)
+                me.event_loop.call_later(1, self.terminate, True)
+            else:
+                KillableThread.terminate(self)
