@@ -1,12 +1,30 @@
-'''Configuration utilities. originally from gunicorn_,
-adapted and modified for pulsar.
+'''Configuration utilities which provides pulsar with configuration parameters
+which can be parsed from the command line. Parsing is implemented using
+the python argparser_ standard library module.
 
-Original Gunicorn Licence
+Config
+~~~~~~~~~~
 
-This file is part of gunicorn released under the MIT license.
-See the NOTICE for more information.
+.. autoclass:: Config
+   :members:
+   :member-order: bysource
 
-.. _gunicorn: http://gunicorn.org/
+SettingBase
+~~~~~~~~~~~~~~
+
+.. autoclass:: SettingBase
+   :members:
+   :member-order: bysource
+      
+Setting
+~~~~~~~~~~
+
+.. autoclass:: Setting
+   :members:
+   :member-order: bysource
+
+
+.. _argparser: http://docs.python.org/dev/library/argparse.html
 '''
 import copy
 import inspect
@@ -14,6 +32,7 @@ import argparse
 import os
 import textwrap
 import types
+import logging
 
 from pulsar import __version__, SERVER_NAME, parse_address
 from . import system
@@ -25,7 +44,7 @@ from .importer import import_system_file
 __all__ = ['Config',
            'SimpleSetting',
            'Setting',
-           'defaults',
+           #'defaults',
            'ordered_settings',
            'validate_string',
            'validate_callable',
@@ -33,7 +52,9 @@ __all__ = ['Config',
            'validate_list',
            'validate_pos_int',
            'validate_pos_float',
-           'make_settings']
+           'make_optparse_options']
+
+LOGGER = logging.getLogger('pulsar.config')
 
 class DefaultSettings:
 
@@ -50,14 +71,14 @@ class DefaultSettings:
         self.message_timeout = 3
 
 defaults = DefaultSettings()
+section_docs = {}
 KNOWN_SETTINGS = {}
 KNOWN_SETTINGS_ORDER = []
 
 
-def def_arity1(server):
-    pass
-
-def def_arity2(worker, req):
+def pass_through(arg):
+    '''A dummy function accepting on parameter only. It does nothing. It is
+used as default by :ref:`Application Hooks <setting-section-application-hooks>`.'''
     pass
 
 def wrap_method(func):
@@ -65,55 +86,58 @@ def wrap_method(func):
         return func(*args, **kwargs)
     return _wrapped
 
-
 def ordered_settings():
     for name in KNOWN_SETTINGS_ORDER:
         yield KNOWN_SETTINGS[name]
 
-def make_settings(apps=None, include=None, exclude=None):
-    '''Creates a dictionary of available settings for given
-applications *apps*.
-
-:parameter apps: Optional list of application names.
-:parameter include: Optional list of settings to include.
-:parameter exclude: Optional list of settings to exclude.
-:rtype: dictionary of :class:`pulsar.Setting` instances.'''
-    settings = {}
-    include = set(include or ())
-    exclude = set(exclude or ())
-    apps = set(apps or ())
-    for s in ordered_settings():
-        setting = s()
-        if setting.name not in include:
-            if setting.name in exclude:
-                continue    # setting name in exclude set
-            if setting.app and setting.app not in apps:
-                continue    # the setting is for an app not in the apps set
-        settings[setting.name] = setting.copy()
-    return settings
-
 
 class Config(object):
-    '''Dictionary containing :class:`Setting` parameters for
+    '''A dictionary-like container of :class:`Setting` parameters for
 fine tuning pulsar servers. It provides easy access to :attr:`Setting.value`
 attribute by exposing the :attr:`Setting.name` as attribute.
 
+:param description: description used when parsing the command line, same usage
+    as in the :class:`argparse.ArgumentParser` class.
+:param epilog: epilog used when parsing the command line, same usage
+    as in the :class:`argparse.ArgumentParser` class.
+:param version: version used when parsing the command line, same usage
+    as in the :class:`argparse.ArgumentParser` class.
+:param apps: list of application namespaces to include in the :attr:`settings`
+    attribute.
+
 .. attribute:: settings
 
-    Dictionary of all :class:`Settings` instances available. The
-    keys are given by the :attr:`Setting.name` attribute.
+    Dictionary of all :class:`Setting` instances available in this
+    :class:`Config` container. Keys are given by the :attr:`SettingBase.name`
+    attribute.
+    
+.. attribute:: params
+
+    Dictionary of additional parameters which cannot be parsed in the
+    command line.
 '''
     exclude_from_config = set(('config',))
     
     def __init__(self, description=None, epilog=None,
-                 version=None, app=None, include=None,
-                 exclude=None, settings=None):
+                 version=None, apps=None, include=None,
+                 exclude=None, settings=None, prefix=None,
+                 name=None, **params):
         if settings is None:
-            settings = make_settings(app, include, exclude)
-        self.settings = settings
+            self.settings = {}
+        else:
+            self.settings = settings
+        self.name = name
+        self.prefix = prefix
+        self.include = set(include or ())
+        self.exclude = set(exclude or ())
+        self.apps = set(apps or ())
+        if settings is None:
+            self.update_settings()
+        self.params = {}
         self.description = description or 'Pulsar server'
         self.epilog = epilog or 'Have fun!'
         self.version = version or __version__
+        self.update(params)
         
     def __iter__(self):
         return iter(self.settings)
@@ -121,6 +145,9 @@ attribute by exposing the :attr:`Setting.name` as attribute.
     def __len__(self):
         return len(self.settings)
     
+    def __contains__(self, name):
+        return name in self.settings
+    
     def items(self):
         for k, setting in iteritems(self.settings):
             yield k, setting.value
@@ -129,8 +156,11 @@ attribute by exposing the :attr:`Setting.name` as attribute.
         return self.__dict__.copy()
 
     def __setstate__(self, state):
-        for k,v in state.items():
+        for k, v in state.items():
             self.__dict__[k] = v
+        config = getattr(self, 'config', None)
+        if config:
+            self.import_from_module(config)
 
     def __getattr__(self, name):
         return self.get(name)
@@ -140,25 +170,49 @@ attribute by exposing the :attr:`Setting.name` as attribute.
             raise AttributeError("Invalid access!")
         super(Config, self).__setattr__(name, value)
 
+    def update(self, data):
+        '''Update this :attr:`Config` with ``data`` which is either an
+instance of Mapping or :class:`Config`.'''
+        for name, value in data.items():
+            if value is not None:
+                self.set(name, value)
+            
     def get(self, name, default=None):
+        '''Get the value at ``name`` for this :class:`Config` container
+following this algorithm:
+
+* check ``name`` in the :attr:`settings` dictionary.
+* if ``name`` is a known setting parameter return ``default``
+* check ``name`` in the :attr:`params` dictionary
+* raise :class:`AttributeError`.
+'''
         if name not in self.settings:
             if name in KNOWN_SETTINGS:
                 return default
+            if name in self.params:
+                return self.params[name]
             raise AttributeError("No configuration setting for: %s" % name)
         return self.settings[name].get()
 
-    def set(self, name, value):
+    def set(self, name, value, default=False):
+        '''Set the configuration :class:`Setting` at *name* with a new
+*value*. If the *name* is not in this container, an :class:`AttributeError`
+is raised. If ``default`` is ``True``, the :attr:`Setting.default` is
+also set.'''
         if name not in self.settings:
-            raise AttributeError("No configuration setting for: %s" % name)
-        self.settings[name].set(value)
+            if self.prefix:
+                prefix_name = '%s_%s' % (self.prefix, name)
+                if prefix_name in self.settings:
+                    return # dont set this value
+            self.params[name] = value
+        else:
+            self.settings[name].set(value, default=default)
 
     def parser(self):
         '''Create the argparser_ for this configuration by adding all
-settings via the :meth:`Setting.add_argument`.
+settings via the :meth:`Setting.add_argument` method.
 
 :rtype: an instance of :class:`argparse.ArgumentParser`.
-
-.. _argparser: http://docs.python.org/dev/library/argparse.html
 '''
         kwargs = {
             "description": self.description,
@@ -168,12 +222,15 @@ settings via the :meth:`Setting.add_argument`.
         parser.add_argument('--version',
                             action='version',
                             version=self.version)
+        return self.add_to_parser(parser)
+
+    def add_to_parser(self, parser):
         setts = self.settings
         sorter = lambda x: (setts[x].section, setts[x].order)
         for k in sorted(setts, key=sorter):
             setts[k].add_argument(parser)
         return parser
-
+        
     def import_from_module(self, mod=None):
         if mod:
             self.set('config', mod)
@@ -208,7 +265,10 @@ settings via the :meth:`Setting.add_argument`.
 
     @property
     def address(self):
-        bind = self.settings['bind']
+        '''An address to bind to, only available if a
+:ref:`bind <setting-bind>` setting has been added to this :class:`Config`
+container.'''
+        bind = self.settings.get('bind')
         if bind:
             return parse_address(to_bytes(bind.get()))
 
@@ -236,14 +296,24 @@ settings via the :meth:`Setting.add_argument`.
             if pn:
                 return pn.get()
         
-    def copy(self):
+    def copy(self, name=None, prefix=None):
+        '''A deep copy of this :class:`Config` container. If ``prefix``
+is given, it prefixes all non
+:ref:`global settings <setting-section-global-server-settings>` with it.'''
         cls = self.__class__
         me = cls.__new__(cls)
-        for name, value in iteritems(self.__dict__):
-            if name == 'settings':
-                value = copy.deepcopy(value)
-            me.__dict__[name] = value
-        return me        
+        me.__dict__.update(self.__dict__)
+        if prefix:
+            me.prefix = prefix
+        # Important, don't use the prefix from me.prefix!
+        #        prefix = me.prefix
+        settings = me.settings
+        me.settings = {}
+        for setting in settings.values():
+            setting = setting.copy(name, prefix)
+            me.settings[setting.name] = setting
+        me.params = me.params.copy()
+        return me
     
     def __copy__(self):
         return self.copy()
@@ -251,12 +321,20 @@ settings via the :meth:`Setting.add_argument`.
     def __deepcopy__(self, memo):
         return self.__copy__()
     
-    def new_config(self):
-        cfg = self.__class__()
-        for key, sett in iteritems(self.settings):
-            if key in cfg.settings and sett.inherit:
-                cfg.set(key, sett.value)
-        return cfg
+    ############################################################################
+    ##    INTERNALS
+    def update_settings(self):
+        for s in ordered_settings():
+            setting = s().copy(name=self.name, prefix=self.prefix)
+            if setting.name in self.settings:  
+                continue
+            if setting.name not in self.include:
+                if setting.name in self.exclude:
+                    continue    # setting name in exclude set
+                if setting.app and setting.app not in self.apps:
+                    continue    # the setting is for an app not in the apps set
+            self.settings[setting.name] = setting
+    
             
 
 class SettingMeta(type):
@@ -287,43 +365,66 @@ in the global ``KNOWN_SETTINGS`` list.'''
     def fmt_desc(cls, desc):
         desc = textwrap.dedent(desc).strip()
         setattr(cls, "desc", desc)
-        lines = desc.splitlines()
+        lines = desc.split('\n\n')
         setattr(cls, "short", '' if not lines else lines[0])
 
 
 class SettingBase(object):
-    inherit = True
+    '''This is the base class of :class:`Settings` and :class:`SimpleSetting`.
+It defines attributes for a given setting parameter.'''
     name = None
-    '''The unique name used to access this setting.'''
+    '''The unique name used to access this setting in the :class:`Config`
+    container.'''
     validator = None
-    '''Validator for this setting. It provided it must be a fucntion
-accepting one positional argument, the value to validate.'''
+    '''A validating function for this setting. It provided it must be a
+function accepting one positional argument, the value to validate.'''
     value = None
     '''The actual value for this setting.'''
+    default = None
+    '''The default value for this setting.'''
     def __str__(self):
         return '{0} ({1})'.format(self.name, self.value)
     __repr__ = __str__
 
-    def set(self, val):
-        '''Set *val* as the value in this :class:`Setting`.'''
-        if hasattr(self.validator,'__call__'):
+    def set(self, val, default=False):
+        '''Set *val* as the :attr:`value` for this :class:`SettingBase`.
+If *default* is ``True`` set also the :attr:`default` value.'''
+        if hasattr(self.validator, '__call__'):
             val = self.validator(val)
         self.value = val
+        if default:
+            self.default = val
 
     def get(self):
+        '''Returns :attr:`value`'''
         return self.value
 
     def on_start(self):
         '''Called when pulsar server starts. It can be used to perform
 custom initialization for this :class:`Setting`.'''
         pass
-
+    
+    def copy(self, name=None, prefix=None):
+        '''Copy this :class:`SettingBase`'''
+        setting = copy.copy(self)
+        if prefix and not setting.is_global:
+            flags = setting.flags
+            if flags and flags[-1].startswith('--'):
+                # Prefix a setting
+                setting.orig_name = setting.name 
+                setting.name = '%s_%s' % (prefix, setting.name)
+                setting.flags = ['--%s-%s' % (prefix, flags[-1][2:])]
+        if name and not setting.is_global:
+            setting.short = '%s application. %s' % (name , setting.short)
+        return setting
+    
 
 class SimpleSetting(SettingBase):
 
     def __init__(self, name, value):
         self.name = name
         self.value = value
+        self.orig_name = self.name
 
 
 # This works for Python 2 and Python 3
@@ -335,25 +436,38 @@ on the command line or on a config file.'''
     '''If set to ``True`` the settings won't be loaded and it can be only used
 as base class for other settings.'''
     nargs = None
-    '''For positional arguments. Same usage as argparse.'''
+    '''For positional arguments. Same usage as python :mod:`argparse` module.'''
     app = None
     '''Setting for a specific :class:`Application`.'''
     section = None
-    '''Setting section, used for creating documentation.'''
+    '''Setting section, used for creating the
+    :ref:`settings documentation <settings>`.'''
     flags = None
     '''List of options strings, e.g. ``[-f, --foo]``.'''
     choices = None
     '''Restrict the argument to the choices provided.'''
     type = None
     meta = None
+    '''Same usage as ``metavar`` in the python :mod:`argparse` module. It is
+    the name for the argument in usage message.'''
     action = None
-    default = None
-    '''Default value'''
+    '''Same usage as ``action`` in the python
+    :mod:`argparse.ArgumentParser.add_argument` method.'''
     short = None
     desc = None
+    '''Description string'''
+    is_global = False
+    '''Flag used by pulsar :ref:`application framework <apps-framework>`
+    when multiple application are used in a running server. If ``False``
+    additional settings will be added to the :class:`Config` container.
+    These settings are clones of this :class:`Setting` with
+    name given by each application name and underacsore ``_``
+    prefixing this :attr:`name`.'''
+    orig_name = None
 
     def __init__(self, name=None, flags=None, action=None, type=None,
-                 default=None, nargs=None, desc=None, validator=None):
+                 default=None, nargs=None, desc=None, validator=None,
+                 app=None):
         self.default = default if default is not None else self.default
         if self.default is not None:
             self.set(self.default)
@@ -363,10 +477,9 @@ as base class for other settings.'''
         self.nargs = nargs or self.nargs
         self.type = type or self.type
         self.desc = desc or self.desc
+        self.app = app or self.app
         self.short = self.short or self.desc
         self.desc = self.desc or self.short
-        #if validator:
-        #    self.validator = wrap_method(validator)
         if self.app and not self.section:
             self.section = self.app
         if not self.section:
@@ -379,7 +492,9 @@ as base class for other settings.'''
         return self.__dict__.copy()
 
     def add_argument(self, parser):
-        '''Add itself to the argparser.'''
+        '''Add this :class:`Setting` to the *parser*, an instance of
+python :class:`argparse.ArgumentParser`, only if :attr:`flags` or
+:attr:`nargs` and :attr:`name` are defined.'''
         kwargs = {'nargs':self.nargs}
         if self.type and self.type != 'string':
             kwargs["type"] = self.type
@@ -400,10 +515,7 @@ as base class for other settings.'''
             # Not added to argparser
             return
         parser.add_argument(*args, **kwargs)
-
-    def copy(self):
-        return copy.copy(self)
-
+        
 
 def validate_bool(val):
     if isinstance(val, bool):
@@ -468,106 +580,45 @@ def validate_callable(arity):
     return _validate_callable
 
 
-class ConfigFile(Setting):
+def make_optparse_options(apps=None, exclude=None, include=None): # pragma nocover
+    '''Create a tuple of optparse options'''
+    from optparse import make_option
+    class AddOptParser(list):
+        def add_argument(self, *args, **kwargs):
+            self.append(make_option(*args, **kwargs))
+    config = Config(apps=apps, exclude=None, include=None)
+    parser = AddOptParser()
+    config.add_to_parser(parser)
+    return tuple(parser)
+    
+
+################################################################################
+##    Global Server Settings
+
+section_docs['Global Server Settings'] = '''
+These settings are global in the sense that they are used by the arbiter
+as well as all pulsar workers. They are server configuration parameters.
+'''
+class Global(Setting):
+    virtual = True
+    section = "Global Server Settings"
+    is_global = True
+
+
+class ConfigFile(Global):
     name = "config"
-    section = "Config File"
     flags = ["-c", "--config"]
     meta = "FILE"
     validator = validate_string
     default = 'config.py'
     desc = """\
-        The path to a Pulsar config file.
-
-        Only has an effect when specified on the command line or as part of an
-        application specific configuration.
+        The path to a Pulsar config file, where default Settings
+        paramaters can be specified.
         """
 
-
-class Workers(Setting):
-    inherit = False     # not ineritable by the arbiter
-    name = "workers"
-    section = "Worker Processes"
-    flags = ["-w", "--workers"]
-    validator = validate_pos_int
-    type = int
-    default = 1
-    desc = """\
-        The number of workers for handling requests.
-
-If you are using a multi-process concurrency, a number in the
-the 2-4 x $(NUM_CORES) range should be good. If you are using threads this
-number can be higher."""
-
-
-class Concurrency(Setting):
-    name = "concurrency"
-    section = "Worker Processes"
-    flags = ["--concurrency"]
-    default = "process"
-    desc = """\
-        The type of concurrency to use: ``process`` or ``thread``.
-        """
-
-
-class MaxRequests(Setting):
-    inherit = False     # not ineritable by the arbiter
-    name = "max_requests"
-    section = "Worker Processes"
-    flags = ["--max-requests"]
-    validator = validate_pos_int
-    type = int
-    default = 0
-    desc = """\
-        The maximum number of requests a worker will process before restarting.
-
-        Any value greater than zero will limit the number of requests a work
-        will process before automatically restarting. This is a simple method
-        to help limit the damage of memory leaks.
-
-        If this is set to zero (the default) then the automatic worker
-        restarts are disabled.
-        """
-
-class Backlog(Setting):
-    inherit = False     # not ineritable by the arbiter
-    name = "backlog"
-    section = "Worker Processes"
-    flags = ["--backlog"]
-    validator = validate_pos_int
-    type = int
-    default = 2048
-    desc = """\
-        The maximum number of concurrent requests.
-        This refers to the number of clients that can be waiting to be served.
-        Exceeding this number results in the client getting an error when
-        attempting to connect. It should only affect servers under significant
-        load.
-
-        Must be a positive integer. Generally set in the 64-2048 range.
-        """
-
-
-class Timeout(Setting):
-    inherit = False     # not ineritable by the arbiter
-    name = "timeout"
-    section = "Worker Processes"
-    flags = ["-t", "--timeout"]
-    validator = validate_pos_int
-    type = int
-    default = 30
-    desc = """\
-        Workers silent for more than this many seconds are killed and restarted.
-
-        Generally set to thirty seconds. Only set this noticeably higher if
-        you're sure of the repercussions for sync workers. For the non sync
-        workers it just means that the worker process is still communicating and
-        is not tied to the length of time required to handle a single request.
-        """
-
-
-class HttpProxyServer(Setting):
+    
+class HttpProxyServer(Global):
     name = "http_proxy"
-    section = "Http"
     flags = ["--http-proxy"]
     default = ''
     desc = """\
@@ -579,9 +630,8 @@ class HttpProxyServer(Setting):
             os.environ['https_proxy'] = self.value
 
 
-class HttpParser(Setting):
+class HttpParser(Global):
     name = "http_py_parser"
-    section = "Http"
     flags = ["--http-py-parser"]
     action = "store_true"
     default = False
@@ -591,13 +641,12 @@ class HttpParser(Setting):
 
     def on_start(self):
         if self.value:
-            from pulsar.lib import setDefaultHttpParser 
+            from pulsar.utils.httpurl import setDefaultHttpParser 
             setDefaultHttpParser(PyHttpParser)
 
 
-class Debug(Setting):
+class Debug(Global):
     name = "debug"
-    section = "Debugging"
     flags = ["--debug"]
     validator = validate_bool
     action = "store_true"
@@ -610,9 +659,8 @@ class Debug(Setting):
         """
 
 
-class Daemon(Setting):
+class Daemon(Global):
     name = "daemon"
-    section = "Server Mechanics"
     flags = ["-D", "--daemon"]
     validator = validate_bool
     action = "store_true"
@@ -625,9 +673,8 @@ class Daemon(Setting):
         """
 
 
-class Pidfile(Setting):
+class Pidfile(Global):
     name = "pidfile"
-    section = "Server Mechanics"
     flags = ["-p", "--pid"]
     meta = "FILE"
     validator = validate_string
@@ -639,18 +686,16 @@ class Pidfile(Setting):
         """
 
 
-class Password(Setting):
+class Password(Global):
     name = "password"
-    section = "Server Mechanics"
     flags = ["--password"]
     validator = validate_string
     default = None
     desc = """Set a password for the server"""
 
 
-class User(Setting):
+class User(Global):
     name = "user"
-    section = "Server Mechanics"
     flags = ["-u", "--user"]
     meta = "USER"
     validator = validate_string
@@ -664,9 +709,8 @@ class User(Setting):
         """
 
 
-class Group(Setting):
+class Group(Global):
     name = "group"
-    section = "Server Mechanics"
     flags = ["-g", "--group"]
     meta = "GROUP"
     validator = validate_string
@@ -680,9 +724,8 @@ class Group(Setting):
         """
 
 
-class Loglevel(Setting):
+class Loglevel(Global):
     name = "loglevel"
-    section = "Logging"
     flags = ["--log-level"]
     meta = "LEVEL"
     validator = validate_string
@@ -698,27 +741,16 @@ class Loglevel(Setting):
              * critical
              """
 
-class LogHandlers(Setting):
+class LogHandlers(Global):
     name = "loghandlers"
-    section = "Logging"
     flags = ["--log-handlers"]
     default = ['console']
     validator = validate_list
     desc = """log handlers for pulsar server"""
-    
-    
-class LogEvery(Setting):
-    name = "logevery"
-    section = "Logging"
-    flags = ["--log-every"]
-    validator = validate_pos_int
-    default = 0
-    desc = """Log information every n seconds"""
 
 
-class LogConfig(Setting):
+class LogConfig(Global):
     name = "logconfig"
-    section = "Logging"
     default = {}
     validator = validate_dict
     desc = '''
@@ -728,9 +760,8 @@ class LogConfig(Setting):
     '''
 
 
-class Procname(Setting):
+class Procname(Global):
     name = "process_name"
-    section = "Process Naming"
     flags = ["-n", "--name"]
     meta = "STRING"
     validator = validate_string
@@ -747,76 +778,170 @@ class Procname(Setting):
         """
 
 
-class DefaultProcName(Setting):
+class DefaultProcName(Global):
     name = "default_process_name"
-    section = "Process Naming"
     validator = validate_string
     default = SERVER_NAME
     desc = """\
         Internal setting that is adjusted for each type of application.
         """
 
+################################################################################
+##    Worker Processes
 
-class WhenReady(Setting):
-    name = "when_ready"
-    section = "Server Hooks"
-    validator = validate_callable(1)
-    type = "callable"
-    default = staticmethod(def_arity1)
+section_docs['Worker Processes'] = '''
+This group of configuration parameters control the number of actors
+for a given :class:`pulsar.Monitor`, the type of concurreny of the server and
+other actor-specific parameters.
+'''
+    
+class Workers(Setting):
+    name = "workers"
+    section = "Worker Processes"
+    flags = ["-w", "--workers"]
+    validator = validate_pos_int
+    type = int
+    default = 1
     desc = """\
-        Called just after the server is started.
+        The number of workers for handling requests.
+        
+If using a multi-process concurrency, a number in the
+the ``2-4 x NUM_CORES`` range should be good. If you are using threads this
+number can be higher."""
 
-        The callable needs to accept a single instance variable for the Arbiter.
+
+class Concurrency(Setting):
+    inherit = True # Inherited by the arbiter if an application is first created
+    name = "concurrency"
+    section = "Worker Processes"
+    flags = ["--concurrency"]
+    default = "process"
+    desc = """\
+        The type of concurrency to use: ``process`` or ``thread``.
         """
 
 
-class Prefork(Setting):
-    name = "pre_fork"
-    section = "Server Hooks"
-    validator = validate_callable(1)
-    default = staticmethod(def_arity1)
-    type = "callable"
+class MaxRequests(Setting):
+    name = "max_requests"
+    section = "Worker Processes"
+    flags = ["--max-requests"]
+    validator = validate_pos_int
+    type = int
+    default = 0
     desc = """\
-        Called just before a worker is forked.
+        The maximum number of requests a worker will process before restarting.
 
-        The callable needs to accept two instance variables for the Arbiter and
-        new Worker.
+        Any value greater than zero will limit the number of requests a worker
+        will process before automatically restarting. This is a simple method
+        to help limit the damage of memory leaks.
+
+        If this is set to zero (the default) then the automatic worker
+        restarts are disabled.
+        """
+
+class Backlog(Setting):
+    name = "backlog"
+    section = "Worker Processes"
+    flags = ["--backlog"]
+    validator = validate_pos_int
+    type = int
+    default = 2048
+    desc = """\
+        The maximum number of queued connections in a socket.
+        
+        This refers to the number of clients that can be waiting to be served.
+        Exceeding this number results in the client getting an error when
+        attempting to connect. It should only affect servers under significant
+        load.
+        Must be a positive integer. Generally set in the 64-2048 range.
         """
 
 
+class Timeout(Setting):
+    name = "timeout"
+    section = "Worker Processes"
+    flags = ["-t", "--timeout"]
+    validator = validate_pos_int
+    type = int
+    default = 30
+    desc = """\
+        Workers silent for more than this many seconds are
+        killed and restarted."""
+
+
+################################################################################
+##    APPLICATION HOOKS
+
+section_docs['Application Hooks'] = '''
+Application hooks are functions which can be specified in a
+:ref:`config <setting-config>` file to perform custom tasks in a pulsar server.
+These tasks can be scheduled when events occurs or at every event loop of
+the various components of a pulsar application.
+
+All application hooks are functions which accept one parameter only, the actor
+invoking the function.'''
+    
 class Postfork(Setting):
     name = "post_fork"
-    section = "Server Hooks"
+    section = "Application Hooks"
     validator = validate_callable(1)
     type = "callable"
-    default = staticmethod(def_arity1)
+    default = staticmethod(pass_through)
     desc = """\
         Called just after a worker has been forked.
 
-        The callable needs to accept two instance variables for the Arbiter and
-        new Worker.
+        The event loop is not yet available.
+        """
+        
+
+class WhenReady(Setting):
+    name = "when_ready"
+    section = "Application Hooks"
+    validator = validate_callable(1)
+    type = "callable"
+    default = staticmethod(pass_through)
+    desc = """\
+        Called just before a worker starts its event loop.
+        
+        This is a chance to setup :class:`pulsar.EventLoop` callbacks which
+        can run periodically, at every loop or when some defined events occur.
         """
 
 
-class PreExec(Setting):
-    name = "pre_exec"
-    section = "Server Hooks"
+class ConnectionMade(Setting):
+    name = "connection_made"
+    section = "Application Hooks"
     validator = validate_callable(1)
     type = "callable"
-    default = staticmethod(def_arity1)
+    default = staticmethod(pass_through)
     desc = """\
-        Called just before a new master process is forked.
+        Called after a new connection is made.
 
-        The callable needs to accept a single instance variable for the Arbiter.
+        The callable needs to accept one instance variables for the
+        connection instance.
+        """
+
+
+class ConnectionLost(Setting):
+    name = "connection_lost"
+    section = "Application Hooks"
+    validator = validate_callable(1)
+    type = "callable"
+    default = staticmethod(pass_through)
+    desc = """
+        Called after a connection is lost.
+
+        The callable needs to accept one instance variables for the
+        connection instance.
         """
 
 
 class PreRequest(Setting):
     name = "pre_request"
-    section = "Server Hooks"
-    validator = validate_callable(2)
+    section = "Application Hooks"
+    validator = validate_callable(1)
     type = "callable"
-    default = staticmethod(def_arity2)
+    default = staticmethod(pass_through)
     desc = """\
         Called just before a worker processes the request.
 
@@ -827,50 +952,13 @@ class PreRequest(Setting):
 
 class PostRequest(Setting):
     name = "post_request"
-    section = "Server Hooks"
-    validator = validate_callable(2)
+    section = "Application Hooks"
+    validator = validate_callable(1)
     type = "callable"
-    default = staticmethod(def_arity2)
+    default = staticmethod(pass_through)
     desc = """\
         Called after a worker processes the request.
 
         The callable needs to accept two instance variables for the Worker and
         the Request.
         """
-
-
-class WorkerExit(Setting):
-    name = "worker_exit"
-    section = "Server Hooks"
-    validator = validate_callable(1)
-    type = "callable"
-    default = staticmethod(def_arity1)
-    desc = """Called just after a worker has been exited.
-
-        The callable needs to accept one variable for the
-        the just-exited Worker.
-        """
-
-
-class WorkerTask(Setting):
-    name = "worker_task"
-    section = "Server Hooks"
-    validator = validate_callable(1)
-    type = "callable"
-    default = staticmethod(def_arity1)
-    desc = """Called at every event loop by the worker.
-
-        The callable needs to accept one variable for the Worker.
-        """
-        
-        
-class ArbiterTask(Setting):
-    name = "arbiter_task"
-    section = "Server Hooks"
-    validator = validate_callable(1)
-    type = "callable"
-    default = staticmethod(def_arity1)
-    desc = """Called at every event loop by the arbiter.
-
-        The callable needs to accept one variable for the Arbiter.
-        """
\ No newline at end of file
