@@ -2,11 +2,9 @@
 import os
 import sys
 
-if sys.version_info > (3,0):
-    string_type = str
-else:   #pragma    nocover
-    string_type = unicode
-    range = xrange
+from .importer import import_module
+from .pep import range, string_type
+
 
 __all__ = ['Path']
 
@@ -16,7 +14,7 @@ class Path(string_type):
     def __new__(cls, path=None):
         path = path or ''
         abspath = os.path.abspath(path)
-        return super(Path,cls).__new__(cls, abspath)
+        return super(Path, cls).__new__(cls, abspath)
 
     @classmethod
     def cwd(cls):
@@ -33,7 +31,7 @@ class Path(string_type):
     
     def split(self):
         d,f = os.path.split(self)
-        return self.__class__(d),f
+        return self.__class__(d), f
     
     def dir(self):
         if self.isfile():
@@ -42,11 +40,24 @@ class Path(string_type):
             return self
         else:
             raise ValueError('%s not a valid directory' % self)
-        
+    
+    @property
+    def basename(self):
+        return os.path.basename(self) 
+    
     @property
     def parent(self):
         return self.__class__(os.path.dirname(self))
         
+    def module_name(self):
+        name = os.path.basename(self)
+        if name.endswith('.py'):
+            return name[:-3]
+        if name.endswith('.pyc'):
+            return name[:-4]
+        else:
+            raise ValueError('%s not a valid python module' % self)
+        
     def ancestor(self, n):
         p = self
         for i in range(n):
@@ -60,7 +71,7 @@ class Path(string_type):
             return self.endswith('.py')
              
     def add2python(self, module=None, up=0, down=None, front=False,
-                   must_exist=True):
+                    must_exist=True):
         '''Add a directory to the python path.
         
 :parameter module: Optional module name to try to import once we have found
@@ -74,28 +85,25 @@ class Path(string_type):
 :parameter must_exist: Boolean indicating if the module must exists.'''
         if module:
             try:
-                __import__(module)
-                return module
+                return import_module(module)
             except ImportError:
                 pass
-        
         dir = self.dir().ancestor(up)
         if down:
             dir = dir.join(*down)
-        added = False
         if dir.isdir():
             if dir not in sys.path:
                 if front:
                     sys.path.insert(0, dir)
                 else:
                     sys.path.append(dir)
-                added = True
+        elif must_exist:
+            raise ImportError('Directory {0} not available'.format(dir))
         else:
-            raise ValueError('Directory {0} not available'.format(dir))
+            return None
         if module:
             try:
-                __import__(module)
-                return module
+                return import_module(module)
             except ImportError:
                 if must_exist:
                     raise
